
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;


entity operativeUnit is
    port (
        I_clk       : in  std_logic;
        I_pixel     : in  std_logic_vector (7 downto 0);  -- Pixel from the memory
        I_ldPix11   : in  std_logic;
        I_ldPix21   : in  std_logic;
        I_ldPix31   : in  std_logic;
        I_shReg     : in  std_logic;
        I_ldPixEdge : in  std_logic;
        O_pixEdge   : out std_logic
        );
end operativeUnit;


architecture Behavioral of operativeUnit is

-- déclaration des sous-composants

    -- banc de registres
    component regUnit is
        port (I_clk                     : in  std_logic;
              I_pixel                   : in  std_logic_vector (7 downto 0);  -- Pixel from the memory
              I_ldPix11                 : in  std_logic;
              I_ldPix21                 : in  std_logic;
              I_ldPix31                 : in  std_logic;
              I_shReg                   : in  std_logic;
              O_Pix11, O_Pix12, O_Pix13 : out std_logic_vector (7 downto 0);
              O_Pix21, O_Pix22, O_Pix23 : out std_logic_vector (7 downto 0);
              O_Pix31, O_Pix32, O_Pix33 : out std_logic_vector (7 downto 0)
              );
    end component regUnit;

    -- unité de calcul du gradient
    component gradientUnit is
        port (I_Pix11, I_Pix12, I_Pix13 : in  std_logic_vector (7 downto 0);
              I_Pix21, I_Pix22, I_Pix23 : in  std_logic_vector (7 downto 0);
              I_Pix31, I_Pix32, I_Pix33 : in  std_logic_vector (7 downto 0);
              O_pixEdge                 : out std_logic
              );
    end component gradientUnit;

    -- registre de sortie
    component pixedgeReg is
        port (I_clk : in  std_logic;
              I_pixEdge    : in  std_logic;
              I_ldPixEdge  : in  std_logic;
              O_pixEdge    : out std_logic
              );
    end component pixedgeReg;

-- déclaration des signaux internes
    signal S_Pix11, S_Pix12, S_Pix13 : std_logic_vector (7 downto 0);
    signal S_Pix21, S_Pix22, S_Pix23 : std_logic_vector (7 downto 0);
    signal S_Pix31, S_Pix32, S_Pix33 : std_logic_vector (7 downto 0);
    signal S_pixEdge                 : std_logic;

begin

-- instanciation des sous-composants et établissement des interconnexions

    -- instanciation d'un banc de registres
    regUnit_1 : entity work.regUnit
        port map (
            I_clk     => I_clk,
            I_pixel   => I_pixel,
            I_ldPix11 => I_ldPix11,
            I_ldPix21 => I_ldPix21,
            I_ldPix31 => I_ldPix31,
            I_shReg   => I_shReg,
            O_Pix11   => S_Pix11,
            O_Pix12   => S_Pix12,
            O_Pix13   => S_Pix13,
            O_Pix21   => S_Pix21,
            O_Pix22   => S_Pix22,
            O_Pix23   => S_Pix23,
            O_Pix31   => S_Pix31,
            O_Pix32   => S_Pix32,
            O_Pix33   => S_Pix33);

    -- instanciation d'une unité de calcul du gradient
    gradientUnit_1 : entity work.gradientUnit
        port map (
            O_pixEdge => S_pixEdge,
            I_Pix11   => S_Pix11,
            I_Pix12   => S_Pix12,
            I_Pix13   => S_Pix13,
            I_Pix21   => S_Pix21,
            I_Pix22   => S_Pix22,
            I_Pix23   => S_Pix23,
            I_Pix31   => S_Pix31,
            I_Pix32   => S_Pix32,
            I_Pix33   => S_Pix33);

    -- instanciation d'un registre de sortie
    pixedgeReg_1 : entity work.pixedgeReg
        port map (
            I_clk       => I_clk,
            I_pixEdge   => S_pixEdge,
            I_ldPixEdge => I_ldPixEdge,
            O_pixEdge   => O_pixEdge);

end Behavioral;
